
/**
 * A class describing a transaction
 *
 * @author  Jyrki Nummenmaa
 * @version 0.1
 * @see     LockManager
 */


public class Transaction {

    public static final int ERROR = 0;
    public static final int ACTIVE = 1;
    public static final int WAITING = 2;
    public static final int COMMITTED = 3;
    public static final int ROLLED_BACK = 4;

    /** The transaction id */
    private long txnId;
    /** The transaction status */
    private int txnStatus;
    /** Yes-votes collected */
    private boolean[] hasVotedYes;
    /** Number of servers */
    private int noServers;

    public Transaction(long txnId, int txnStatus, int noServers, int myServerNo) {
	this.txnId = txnId;
	setStatus(txnStatus);
	hasVotedYes = new boolean[noServers];
	for (int i=0;i<noServers;i++)
		 hasVotedYes[myServerNo] = true;		
    }

    public void setStatus(int txnStatus) {
	if ((txnStatus==ACTIVE) || (txnStatus==WAITING) || 
	    (txnStatus==COMMITTED) || (txnStatus==ROLLED_BACK))
	    this.txnStatus = txnStatus;
	else
	    this.txnStatus = ERROR;
    }

    public long getId() {
	return txnId;
    }
    
    public int getStatus() {
	return txnStatus;
    }

    public void setYesVote(int serverNo) {
	hasVotedYes[serverNo]=true;
    }

    public boolean allYes() {
	for (int i=0;i<noServers;i++)
	    if (hasVotedYes[i]==false) {
		return false;
	    }
	return true;
    }

   
    public String outputString() {
	if (txnStatus==ERROR)
	    return("Transaction " + txnId + " has ERROR status.");
	else if (txnStatus==WAITING)
	    return("Transaction " + txnId + " is WAITING.");
	else if (txnStatus==COMMITTED)
	    return("Transaction " + txnId + " has COMMITTED.");
	else if (txnStatus==ROLLED_BACK)
	    return("Transaction " + txnId + " has been ROLLED_BACK.");
	else return("Transaction " + txnId + " has ERROR status.");
    }
}

