import java.net.*;
import java.io.*;
import java.util.*;

/**
 * A thread to send messages to all servers.
 *
 * @author  Jyrki Nummenmaa
 * @version 0.1
 */

public class TwoPCCoordThread extends Thread {
    private LockManager lm = null;
    private boolean[] isConnected;
    private int noServers;
    private int myServerNo;
    private long txnId;
    private boolean done;
    private boolean[] hasVotedYes;

    /** Sockets for server connections */
    Socket[] serverSockets;
    /** PrintWriters for output to servers */
    PrintWriter[] serverOuts;
    /** BufferedReaders for reading input in */
    BufferedReader[] serverIns;

    String fromServer = "";


    Random random;

    public TwoPCCoordThread(LockManager lm, long txnId) {
	super("LMMultiServerThread");
	done = false;
	this.lm = lm;
	this.txnId = txnId;
	noServers = lm.getNoServers();
	myServerNo = lm.getMyServerNo();
	isConnected = new boolean[noServers];
	serverSockets = new Socket[noServers];
        serverOuts = new PrintWriter[noServers];
        serverIns = new BufferedReader[noServers];
	hasVotedYes = new boolean[noServers];
	for (int i=0;i<noServers;i++)
	    hasVotedYes[i]=false;
	hasVotedYes[lm.getMyServerNo()]=true;
	for (int i=0;i<noServers;i++)
	    isConnected[i]=false;
	isConnected[lm.getMyServerNo()]=true;
    }

    private boolean allConnected() {
	for (int i=0;i<noServers;i++)
	    if (isConnected[i]==false)
		return false;
	return true;
    }


    public void run() {

	try {
	    // Get the txn id.
	    Transaction txn = lm.getTxn(txnId);

	    // First establish a connection with all servers.
	    // This may take some time.
	    if (lm.getDebug()>1)
		System.out.println("-Initiating voting.");
	    isConnected[myServerNo]=true;
	    while (!allConnected()) {
		sleep(100);
		for (int i=0;i<noServers;i++)
		    if (!isConnected[i])
			try {	
			    if (lm.getDebug()>1)
				System.out.println("-Trying to connect " + lm.getServerNames()[i] +
						   ":" + lm.getServerPorts()[i]);
			    serverSockets[i] = new Socket(lm.getServerNames()[i], lm.getServerPorts()[i]);
			    serverOuts[i] = new PrintWriter(serverSockets[i].getOutputStream(), true);
			    serverIns[i] = new BufferedReader(new InputStreamReader(serverSockets[i].getInputStream()));
			    isConnected[i]=true;
			}
			catch (Exception e) {}
	    }

	    // Now we have all connections. Start sending messages.

	    if (lm.getDebug()>1)
		System.out.println("-Got connections for voting.");


	    if (!lm.canCommit(txnId)) {
		done = true;
		lm.rollback(txnId);
		for (int i=0;i<noServers;i++) {
		    if (i!=myServerNo) {
			serverOuts[i].println(LockManagerProtocol.TWO_PC_ABORT_COMMAND + " " + txnId);
		    }
		}
	    }
	    else {
		done = false;
		txn.setYesVote(myServerNo);
		lm.setTxnStatus(txnId, Transaction.WAITING);
		for (int i=0;i<noServers;i++)
		    if (i!=myServerNo) {
			serverOuts[i].println(LockManagerProtocol.TWO_PC_VOTE_REQ_COMMAND + " " + txnId);
		}
	    }	    
	    
       	    while (!done) {
		sleep(200);		
		for (int i=0;i<noServers;i++) {
		    if (i!=myServerNo)
			if (serverIns[i].ready()) {
			    fromServer = serverIns[i].readLine();
			    if (lm.getDebug()>1)
				System.out.println("-Got '" + fromServer + "' from server " + i);
			    if (fromServer.equals(LockManagerProtocol.TWO_PC_NO_COMMAND)) {
				done = true;
				lm.rollback(txnId);
				for (int j=0;j<noServers;j++) {
				    if (j!=myServerNo)
					serverOuts[j].println(LockManagerProtocol.TWO_PC_ABORT_COMMAND + " " +
							      txnId);
				}
			    } 
			    else if (fromServer.equals(LockManagerProtocol.TWO_PC_YES_COMMAND)) {
				txn.setYesVote(i);
				if (txn.allYes()) {
				    lm.commit(txnId);
				    done = true;
				    for (int j=0;j<noServers;j++) {
					if (j!=myServerNo)
					    serverOuts[j].println(LockManagerProtocol.TWO_PC_COMMIT_COMMAND + " " +
								  txnId);
				    }
				}
			    }
			}
		}
				    
	    }

	    for (int i=0;i<noServers;i++)
		if (i!=myServerNo) 
		    serverOuts[i].println(LockManagerProtocol.BYE_COMMAND);
		
	} catch (Exception e) {
	    e.printStackTrace();
	    for (int i=0;i<noServers;i++)
		try {
		    if (i!=myServerNo) {
			serverOuts[i].close();
			serverSockets[i].close();
		    }
		}
		catch (Exception ee) {}
	}		    
    }
}
